;/**
; *******************************************************************************
; * @file    startup_TMPM384.s
; * @brief   CMSIS Cortex-M3 Core Device Startup File for the
; *          TOSHIBA 'TMPM384' Device Series
; * @version V2.0.2.2
; * @date    2014/05/20
; *------- <<< Use Configuration Wizard in Context Menu >>> ------------------
; * 
; * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
; * 
; * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
; *******************************************************************************
; */

; <h> Stack Configuration
;   <o> Stack Size (in Bytes) <0x0-0xFFFFFFFF:8>
; </h>

Stack_Size      EQU     0x00000200

                AREA    STACK, NOINIT, READWRITE, ALIGN=3
Stack_Mem       SPACE   Stack_Size
__initial_sp


; <h> Heap Configuration
;   <o>  Heap Size (in Bytes) <0x0-0xFFFFFFFF:8>
; </h>

Heap_Size       EQU     0x00000000

                AREA    HEAP, NOINIT, READWRITE, ALIGN=3
__heap_base
Heap_Mem        SPACE   Heap_Size
__heap_limit


                PRESERVE8
                THUMB


; Vector Table Mapped to Address 0 at Reset

                AREA    RESET, DATA, READONLY
                EXPORT  __Vectors

__Vectors       DCD     __initial_sp              ; Top of Stack
                DCD     Reset_Handler             ; Reset Handler
                DCD     NMI_Handler               ; NMI Handler
                DCD     HardFault_Handler         ; Hard Fault Handler
                DCD     MemManage_Handler         ; MPU Fault Handler
                DCD     BusFault_Handler          ; Bus Fault Handler
                DCD     UsageFault_Handler        ; Usage Fault Handler
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     SVC_Handler               ; SVCall Handler
                DCD     DebugMon_Handler          ; Debug Monitor Handler
                DCD     0                         ; Reserved
                DCD     PendSV_Handler            ; PendSV Handler
                DCD     SysTick_Handler           ; SysTick Handler

                ; External Interrupts
                DCD     INT0_IRQHandler           ; 0:	Interrupt pin (PH0/AIN0/INT0)
                DCD     INT1_IRQHandler           ; 1:	Interrupt pin (PH1/AIN1/INT1)
                DCD     INT2_IRQHandler           ; 2:	Interrupt pin (PH2/AIN2/INT2)
                DCD     INT3_IRQHandler           ; 3:	Interrupt pin (PA0/TB0IN/INT3)
                DCD     INT4_IRQHandler           ; 4:	Interrupt pin (PA2/TB1IN/INT4)
                DCD     INT5_IRQHandler           ; 5:	Interrupt pin (PE4/TB2IN/INT5)
                DCD     INTRX0_IRQHandler         ; 6:	Serial reception (channel.0)
                DCD     INTTX0_IRQHandler         ; 7:	Serial transmission (channel.0)
                DCD     INTRX1_IRQHandler         ; 8:	Serial reception (channel.1)
                DCD     INTTX1_IRQHandler         ; 9:	Serial transmission (channel.1)
                DCD     INTSSP0_IRQHandler        ; 10:	Syncronous Serial Port 0
                DCD     INTSSP1_IRQHandler        ; 11:	Syncronous Serial Port 1
                DCD     INTEMG0_IRQHandler        ; 12:	PMD0 EMG interrupt (MPT0)
                DCD     INTEMG1_IRQHandler        ; 13:	PMD1 EMG interrupt (MPT1)
                DCD     INTSBI0_IRQHandler        ; 14:	Serial Bus Interface 0 interrupt
                DCD     INTSBI1_IRQHandler        ; 15:	Serial Bus Interface 1 interrupt
                DCD     INTADPD0_IRQHandler       ; 16:	ADC conversion triggered by PMD0 is finished
                DCD     INTRTC_IRQHandler         ; 17:	Realtime clock interrupt
                DCD     INTADPD1_IRQHandler       ; 18:	ADC conversion triggered by PMD1 is finished
                DCD     INTRMCRX_IRQHandler       ; 19:	Remote Controller reception interrupt
                DCD     INTTB00_IRQHandler        ; 20:	16bit TMRB0 compare match detection 0
                DCD     INTTB01_IRQHandler        ; 21:	16bit TMRB0 compare match detection 1
                DCD     INTTB10_IRQHandler        ; 22:	16bit TMRB1 compare match detection 0
                DCD     INTTB11_IRQHandler        ; 23:	16bit TMRB1 compare match detection 1
                DCD     INTTB40_IRQHandler        ; 24:	16bit TMRB4 compare match detection 0
                DCD     INTTB41_IRQHandler        ; 25:	16bit TMRB4 compare match detection 1
                DCD     INTTB50_IRQHandler        ; 26:	16bit TMRB5 compare match detection 0
                DCD     INTTB51_IRQHandler        ; 27:	16bit TMRB5 compare match detection 1
                DCD     INTPMD0_IRQHandler        ; 28:	PMD0 PWM interrupt (MPT0)
                DCD     INTPMD1_IRQHandler        ; 29:	PMD1 PWM interrupt (MPT1)
                DCD     INTCAP00_IRQHandler       ; 30:	16bit TMRB0 input capture 0
                DCD     INTCAP01_IRQHandler       ; 31:	16bit TMRB0 input capture 1
                DCD     INTCAP10_IRQHandler       ; 32:	16bit TMRB1 input capture 0
                DCD     INTCAP11_IRQHandler       ; 33:	16bit TMRB1 input capture 1
                DCD     INTCAP40_IRQHandler       ; 34:	16bit TMRB4 input capture 0
                DCD     INTCAP41_IRQHandler       ; 35:	16bit TMRB4 input capture 1
                DCD     INTCAP50_IRQHandler       ; 36:	16bit TMRB5 input capture 0
                DCD     INTCAP51_IRQHandler       ; 37:	16bit TMRB5 input capture 1
                DCD     INT6_IRQHandler           ; 38:	Interrupt Pin (PE6/TB3IN/INT6)
                DCD     INT7_IRQHandler           ; 39:	Interrupt Pin (PE7/TB3OUT/INT7)
                DCD     INTRX2_IRQHandler         ; 40:	Serial reception (channel.2)
                DCD     INTTX2_IRQHandler         ; 41:	Serial transmit (channel.2)
                DCD     INTADCP0_IRQHandler       ; 42:	ADC conversion monitoring function interrupt 0
                DCD     INTADCP1_IRQHandler       ; 43:	ADC conversion monitoring function interrupt 1
                DCD     INTRX4_IRQHandler         ; 44:	Serial reception (channel.4)
                DCD     INTTX4_IRQHandler         ; 45:	Serial transmit (channel.4)
                DCD     INTTB20_IRQHandler        ; 46:	16bit TMRB2 compare match detection 0
                DCD     INTTB21_IRQHandler        ; 47:	16bit TMRB2 compare match detection 1
                DCD     INTTB30_IRQHandler        ; 48:	16bit TMRB3 compare match detection 0
                DCD     INTTB31_IRQHandler        ; 49:	16bit TMRB3 compare match detection 1
                DCD     INTCAP20_IRQHandler       ; 50:	16bit TMRB2 input capture 0
                DCD     INTCAP21_IRQHandler       ; 51:	16bit TMRB2 input capture 1
                DCD     INTCAP30_IRQHandler       ; 52:	16bit TMRB3 input capture 0
                DCD     INTCAP31_IRQHandler       ; 53:	16bit TMRB3 input capture 1
                DCD     INTADSFT_IRQHandler       ; 54:	ADC conversion started by software is finished
                DCD     0                         ; 55: Reserved
                DCD     INTADTMR_IRQHandler       ; 56:	ADC conversion triggered by timer is finished
                DCD     0                         ; 57: Reserved
                DCD     INT8_IRQHandler           ; 58:	Interrupt Pin (PA7/TB4IN/INT8)
                DCD     INT9_IRQHandler           ; 59:	Interrupt Pin (PD3/INT9)
                DCD     INTA_IRQHandler           ; 60:	Interrupt Pin (PJ6/AIN6/INTA)
                DCD     INTB_IRQHandler           ; 61:	Interrupt Pin (PJ7/AIN7/INTB)
                DCD     INTENC0_IRQHandler        ; 62:	Encoder input0 interrupt
                DCD     INTENC1_IRQHandler        ; 63:	Encoder input1 interrupt
                DCD     INTRX3_IRQHandler         ; 64:	Serial reception (channel.3)
                DCD     INTTX3_IRQHandler         ; 65:	Serial transmit (channel.3)
                DCD     INTTB60_IRQHandler        ; 66:	16bit TMRB6 compare match detection 0
                DCD     INTTB61_IRQHandler        ; 67:	16bit TMRB6 compare match detection 1
                DCD     INTTB70_IRQHandler        ; 68:	16bit TMRB7 compare match detection 0
                DCD     INTTB71_IRQHandler        ; 69:	16bit TMRB7 compare match detection 1
                DCD     INTCAP60_IRQHandler       ; 70:	16bit TMRB6 input capture 0
                DCD     INTCAP61_IRQHandler       ; 71:	16bit TMRB6 input capture 1
                DCD     INTCAP70_IRQHandler       ; 72:	16bit TMRB7 input capture 0
                DCD     INTCAP71_IRQHandler       ; 73:	16bit TMRB7 input capture 1
                DCD     INTC_IRQHandler           ; 74:	Interrupt Pin (PD0/ENCA0/TB5IN/INTC)
                DCD     INTD_IRQHandler           ; 75:	Interrupt Pin (PD2/ENCZ0/INTD)
                DCD     INTE_IRQHandler           ; 76:	Interrupt Pin (PN7/MT2IN/INTE)
                DCD     INTF_IRQHandler           ; 77:	Interrupt Pin (PL2/INTF)
                DCD     INTDMACERR_IRQHandler     ; 78:	DMA transfer error
                DCD     INTDMACTC_IRQHandler      ; 79:	DMA end of transfer
                DCD     INTMTTB00_IRQHandler      ; 80:	16-bit MPT0 IGBT period/ compare match detection 0/Over flow
                DCD     INTMTTB01_IRQHandler      ; 81:	16-bit MPT0 IGBT trigger/ compare match detection 1
                DCD     INTMTTB10_IRQHandler      ; 82:	16-bit MPT1 IGBT period/ compare match detection 0/Over flow
                DCD     INTMTTB11_IRQHandler      ; 83:	16-bit MPT1 IGBT trigger/ compare match detection 1
                DCD     INTMTTB20_IRQHandler      ; 84:	16-bit MPT2 IGBT period/ compare match detection 0/Over flow
                DCD     INTMTTB21_IRQHandler      ; 85:	16-bit MPT2 IGBT trigger/ compare match detection 1
                DCD     INTMTCAP00_IRQHandler     ; 86:	16-bit MPT0 input capture 0
                DCD     INTMTCAP01_IRQHandler     ; 87:	16-bit MPT0 input capture 1
                DCD     INTMTCAP10_IRQHandler     ; 88:	16-bit MPT1 input capture 0
                DCD     INTMTCAP11_IRQHandler     ; 89:	16-bit MPT1 input capture 1
                DCD     INTMTCAP20_IRQHandler     ; 90:	16-bit MPT2 input capture 0
                DCD     INTMTCAP21_IRQHandler     ; 91:	16-bit MPT2 input capture 1
                DCD     INTMTEMG0_IRQHandler      ; 92:	16-bit MPT0 IGBT EMG interrupt
                DCD     INTMTEMG1_IRQHandler      ; 93:	16-bit MPT1 IGBT EMG interrupt
                DCD     INTMTEMG2_IRQHandler      ; 94:	16-bit MPT2 IGBT EMG interrupt
                DCD     INTTB80_IRQHandler        ; 95:	16bit TMRB8 compare match detection 0
                DCD     INTTB81_IRQHandler        ; 96:	16bit TMRB8 compare match detection 1
                DCD     INTTB90_IRQHandler        ; 97:	16bit TMRB9 compare match detection 0
                DCD     INTTB91_IRQHandler        ; 98:	16bit TMRB9 compare match detection 1
                DCD     INTTBA0_IRQHandler        ; 99:	16bit TMRBA compare match detection 0
                DCD     INTTBA1_IRQHandler        ; 100:	16bit TMRBA compare match detection 1
                DCD     INTTBB0_IRQHandler        ; 101:	16bit TMRBB compare match detection 0
                DCD     INTTBB1_IRQHandler        ; 102:	16bit TMRBB compare match detection 1
                DCD     INTCAP80_IRQHandler       ; 103:	16bit TMRB8 input capture 0
                DCD     INTCAP81_IRQHandler       ; 104:	16bit TMRB8 input capture 1
                DCD     INTCAP90_IRQHandler       ; 105:	16bit TMRB9 input capture 0
                DCD     INTCAP91_IRQHandler       ; 106:	16bit TMRB9 input capture 1
                DCD     INTCAPA0_IRQHandler       ; 107:	16bit TMRBA input capture 0
                DCD     INTCAPA1_IRQHandler       ; 108:	16bit TMRBA input capture 1
                DCD     INTCAPB0_IRQHandler       ; 109:	16bit TMRBB input capture 0
                DCD     INTCAPB1_IRQHandler       ; 110:	16bit TMRBB input capture 1
                DCD     INTMTTB30_IRQHandler      ; 111:	16-bit MPT3 IGBT period/ compare match detection 0/Over flow
                DCD     INTMTTB31_IRQHandler      ; 112:	16-bit MPT3 IGBT trigger/ compare match detection 1
                DCD     INTMTCAP30_IRQHandler     ; 113:	16-bit MPT3 input capture 0
                DCD     INTMTCAP31_IRQHandler     ; 114:	16-bit MPT3 input capture 1
                DCD     INTMTEMG3_IRQHandler      ; 115:	16-bit MPT3 IGBT EMG interrupt



                AREA    |.text|, CODE, READONLY


; Reset Handler

Reset_Handler   PROC
                EXPORT  Reset_Handler             [WEAK]
                IMPORT  SystemInit
                IMPORT  __main
                LDR     R0, =SystemInit
                BLX     R0
                LDR     R0, =__main
                BX      R0
                ENDP


; Dummy Exception Handlers (infinite loops which can be modified)

NMI_Handler     PROC
                EXPORT  NMI_Handler               [WEAK]
                B       .
                ENDP
HardFault_Handler\
                PROC
                EXPORT  HardFault_Handler         [WEAK]
                B       .
                ENDP
MemManage_Handler\
                PROC
                EXPORT  MemManage_Handler         [WEAK]
                B       .
                ENDP
BusFault_Handler\
                PROC
                EXPORT  BusFault_Handler          [WEAK]
                B       .
                ENDP
UsageFault_Handler\
                PROC
                EXPORT  UsageFault_Handler        [WEAK]
                B       .
                ENDP
SVC_Handler     PROC
                EXPORT  SVC_Handler               [WEAK]
                B       .
                ENDP
DebugMon_Handler\
                PROC
                EXPORT  DebugMon_Handler          [WEAK]
                B       .
                ENDP
PendSV_Handler  PROC
                EXPORT  PendSV_Handler            [WEAK]
                B       .
                ENDP
SysTick_Handler PROC
                EXPORT  SysTick_Handler           [WEAK]
                B       .
                ENDP

Default_Handler PROC

                EXPORT  INT0_IRQHandler           [WEAK]
                EXPORT  INT1_IRQHandler           [WEAK]
                EXPORT  INT2_IRQHandler           [WEAK]
                EXPORT  INT3_IRQHandler           [WEAK]
                EXPORT  INT4_IRQHandler           [WEAK]
                EXPORT  INT5_IRQHandler           [WEAK]
                EXPORT  INTRX0_IRQHandler         [WEAK]
                EXPORT  INTTX0_IRQHandler         [WEAK]
                EXPORT  INTRX1_IRQHandler         [WEAK]
                EXPORT  INTTX1_IRQHandler         [WEAK]
                EXPORT  INTSSP0_IRQHandler        [WEAK]
                EXPORT  INTSSP1_IRQHandler        [WEAK]
                EXPORT  INTEMG0_IRQHandler        [WEAK]
                EXPORT  INTEMG1_IRQHandler        [WEAK]
                EXPORT  INTSBI0_IRQHandler        [WEAK]
                EXPORT  INTSBI1_IRQHandler        [WEAK]
                EXPORT  INTADPD0_IRQHandler       [WEAK]
                EXPORT  INTRTC_IRQHandler         [WEAK]
                EXPORT  INTADPD1_IRQHandler       [WEAK]
                EXPORT  INTRMCRX_IRQHandler       [WEAK]
                EXPORT  INTTB00_IRQHandler        [WEAK]
                EXPORT  INTTB01_IRQHandler        [WEAK]
                EXPORT  INTTB10_IRQHandler        [WEAK]
                EXPORT  INTTB11_IRQHandler        [WEAK]
                EXPORT  INTTB40_IRQHandler        [WEAK]
                EXPORT  INTTB41_IRQHandler        [WEAK]
                EXPORT  INTTB50_IRQHandler        [WEAK]
                EXPORT  INTTB51_IRQHandler        [WEAK]
                EXPORT  INTPMD0_IRQHandler        [WEAK]
                EXPORT  INTPMD1_IRQHandler        [WEAK]
                EXPORT  INTCAP00_IRQHandler       [WEAK]
                EXPORT  INTCAP01_IRQHandler       [WEAK]
                EXPORT  INTCAP10_IRQHandler       [WEAK]
                EXPORT  INTCAP11_IRQHandler       [WEAK]
                EXPORT  INTCAP40_IRQHandler       [WEAK]
                EXPORT  INTCAP41_IRQHandler       [WEAK]
                EXPORT  INTCAP50_IRQHandler       [WEAK]
                EXPORT  INTCAP51_IRQHandler       [WEAK]
                EXPORT  INT6_IRQHandler           [WEAK]
                EXPORT  INT7_IRQHandler           [WEAK]
                EXPORT  INTRX2_IRQHandler         [WEAK]
                EXPORT  INTTX2_IRQHandler         [WEAK]
                EXPORT  INTADCP0_IRQHandler       [WEAK]
                EXPORT  INTADCP1_IRQHandler       [WEAK]
                EXPORT  INTRX4_IRQHandler         [WEAK]
                EXPORT  INTTX4_IRQHandler         [WEAK]
                EXPORT  INTTB20_IRQHandler        [WEAK]
                EXPORT  INTTB21_IRQHandler        [WEAK]
                EXPORT  INTTB30_IRQHandler        [WEAK]
                EXPORT  INTTB31_IRQHandler        [WEAK]
                EXPORT  INTCAP20_IRQHandler       [WEAK]
                EXPORT  INTCAP21_IRQHandler       [WEAK]
                EXPORT  INTCAP30_IRQHandler       [WEAK]
                EXPORT  INTCAP31_IRQHandler       [WEAK]
                EXPORT  INTADSFT_IRQHandler       [WEAK]
                EXPORT  INTADTMR_IRQHandler       [WEAK]
                EXPORT  INT8_IRQHandler           [WEAK]
                EXPORT  INT9_IRQHandler           [WEAK]
                EXPORT  INTA_IRQHandler           [WEAK]
                EXPORT  INTB_IRQHandler           [WEAK]
                EXPORT  INTENC0_IRQHandler        [WEAK]
                EXPORT  INTENC1_IRQHandler        [WEAK]
                EXPORT  INTRX3_IRQHandler         [WEAK]
                EXPORT  INTTX3_IRQHandler         [WEAK]
                EXPORT  INTTB60_IRQHandler        [WEAK]
                EXPORT  INTTB61_IRQHandler        [WEAK]
                EXPORT  INTTB70_IRQHandler        [WEAK]
                EXPORT  INTTB71_IRQHandler        [WEAK]
                EXPORT  INTCAP60_IRQHandler       [WEAK]
                EXPORT  INTCAP61_IRQHandler       [WEAK]
                EXPORT  INTCAP70_IRQHandler       [WEAK]
                EXPORT  INTCAP71_IRQHandler       [WEAK]
                EXPORT  INTC_IRQHandler           [WEAK]
                EXPORT  INTD_IRQHandler           [WEAK]
                EXPORT  INTE_IRQHandler           [WEAK]
                EXPORT  INTF_IRQHandler           [WEAK]
                EXPORT  INTDMACERR_IRQHandler     [WEAK]
                EXPORT  INTDMACTC_IRQHandler      [WEAK]
                EXPORT  INTMTTB00_IRQHandler      [WEAK]
                EXPORT  INTMTTB01_IRQHandler      [WEAK]
                EXPORT  INTMTTB10_IRQHandler      [WEAK]
                EXPORT  INTMTTB11_IRQHandler      [WEAK]
                EXPORT  INTMTTB20_IRQHandler      [WEAK]
                EXPORT  INTMTTB21_IRQHandler      [WEAK]
                EXPORT  INTMTCAP00_IRQHandler     [WEAK]
                EXPORT  INTMTCAP01_IRQHandler     [WEAK]
                EXPORT  INTMTCAP10_IRQHandler     [WEAK]
                EXPORT  INTMTCAP11_IRQHandler     [WEAK]
                EXPORT  INTMTCAP20_IRQHandler     [WEAK]
                EXPORT  INTMTCAP21_IRQHandler     [WEAK]
                EXPORT  INTMTEMG0_IRQHandler      [WEAK]
                EXPORT  INTMTEMG1_IRQHandler      [WEAK]
                EXPORT  INTMTEMG2_IRQHandler      [WEAK]
                EXPORT  INTTB80_IRQHandler        [WEAK]
                EXPORT  INTTB81_IRQHandler        [WEAK]
                EXPORT  INTTB90_IRQHandler        [WEAK]
                EXPORT  INTTB91_IRQHandler        [WEAK]
                EXPORT  INTTBA0_IRQHandler        [WEAK]
                EXPORT  INTTBA1_IRQHandler        [WEAK]
                EXPORT  INTTBB0_IRQHandler        [WEAK]
                EXPORT  INTTBB1_IRQHandler        [WEAK]
                EXPORT  INTCAP80_IRQHandler       [WEAK]
                EXPORT  INTCAP81_IRQHandler       [WEAK]
                EXPORT  INTCAP90_IRQHandler       [WEAK]
                EXPORT  INTCAP91_IRQHandler       [WEAK]
                EXPORT  INTCAPA0_IRQHandler       [WEAK]
                EXPORT  INTCAPA1_IRQHandler       [WEAK]
                EXPORT  INTCAPB0_IRQHandler       [WEAK]
                EXPORT  INTCAPB1_IRQHandler       [WEAK]
                EXPORT  INTMTTB30_IRQHandler      [WEAK]
                EXPORT  INTMTTB31_IRQHandler      [WEAK]
                EXPORT  INTMTCAP30_IRQHandler     [WEAK]
                EXPORT  INTMTCAP31_IRQHandler     [WEAK]
                EXPORT  INTMTEMG3_IRQHandler      [WEAK]

INT0_IRQHandler
INT1_IRQHandler
INT2_IRQHandler
INT3_IRQHandler
INT4_IRQHandler
INT5_IRQHandler
INTRX0_IRQHandler
INTTX0_IRQHandler
INTRX1_IRQHandler
INTTX1_IRQHandler
INTSSP0_IRQHandler
INTSSP1_IRQHandler
INTEMG0_IRQHandler
INTEMG1_IRQHandler
INTSBI0_IRQHandler
INTSBI1_IRQHandler
INTADPD0_IRQHandler
INTRTC_IRQHandler
INTADPD1_IRQHandler
INTRMCRX_IRQHandler
INTTB00_IRQHandler
INTTB01_IRQHandler
INTTB10_IRQHandler
INTTB11_IRQHandler
INTTB40_IRQHandler
INTTB41_IRQHandler
INTTB50_IRQHandler
INTTB51_IRQHandler
INTPMD0_IRQHandler
INTPMD1_IRQHandler
INTCAP00_IRQHandler
INTCAP01_IRQHandler
INTCAP10_IRQHandler
INTCAP11_IRQHandler
INTCAP40_IRQHandler
INTCAP41_IRQHandler
INTCAP50_IRQHandler
INTCAP51_IRQHandler
INT6_IRQHandler
INT7_IRQHandler
INTRX2_IRQHandler
INTTX2_IRQHandler
INTADCP0_IRQHandler
INTADCP1_IRQHandler
INTRX4_IRQHandler
INTTX4_IRQHandler
INTTB20_IRQHandler
INTTB21_IRQHandler
INTTB30_IRQHandler
INTTB31_IRQHandler
INTCAP20_IRQHandler
INTCAP21_IRQHandler
INTCAP30_IRQHandler
INTCAP31_IRQHandler
INTADSFT_IRQHandler
INTADTMR_IRQHandler
INT8_IRQHandler
INT9_IRQHandler
INTA_IRQHandler
INTB_IRQHandler
INTENC0_IRQHandler
INTENC1_IRQHandler
INTRX3_IRQHandler
INTTX3_IRQHandler
INTTB60_IRQHandler
INTTB61_IRQHandler
INTTB70_IRQHandler
INTTB71_IRQHandler
INTCAP60_IRQHandler
INTCAP61_IRQHandler
INTCAP70_IRQHandler
INTCAP71_IRQHandler
INTC_IRQHandler
INTD_IRQHandler
INTE_IRQHandler
INTF_IRQHandler
INTDMACERR_IRQHandler
INTDMACTC_IRQHandler
INTMTTB00_IRQHandler
INTMTTB01_IRQHandler
INTMTTB10_IRQHandler
INTMTTB11_IRQHandler
INTMTTB20_IRQHandler
INTMTTB21_IRQHandler
INTMTCAP00_IRQHandler
INTMTCAP01_IRQHandler
INTMTCAP10_IRQHandler
INTMTCAP11_IRQHandler
INTMTCAP20_IRQHandler
INTMTCAP21_IRQHandler
INTMTEMG0_IRQHandler
INTMTEMG1_IRQHandler
INTMTEMG2_IRQHandler
INTTB80_IRQHandler
INTTB81_IRQHandler
INTTB90_IRQHandler
INTTB91_IRQHandler
INTTBA0_IRQHandler
INTTBA1_IRQHandler
INTTBB0_IRQHandler
INTTBB1_IRQHandler
INTCAP80_IRQHandler
INTCAP81_IRQHandler
INTCAP90_IRQHandler
INTCAP91_IRQHandler
INTCAPA0_IRQHandler
INTCAPA1_IRQHandler
INTCAPB0_IRQHandler
INTCAPB1_IRQHandler
INTMTTB30_IRQHandler
INTMTTB31_IRQHandler
INTMTCAP30_IRQHandler
INTMTCAP31_IRQHandler
INTMTEMG3_IRQHandler

                B       .

                ENDP


                ALIGN


; User Initial Stack & Heap

                IF      :DEF:__MICROLIB

                EXPORT  __initial_sp
                EXPORT  __heap_base
                EXPORT  __heap_limit

                ELSE

                IMPORT  __use_two_region_memory
                EXPORT  __user_initial_stackheap
__user_initial_stackheap

                LDR     R0, =  Heap_Mem
                LDR     R1, =(Stack_Mem + Stack_Size)
                LDR     R2, = (Heap_Mem +  Heap_Size)
                LDR     R3, = Stack_Mem
                BX      LR

                ALIGN

                ENDIF


                END

