/**
 *******************************************************************************
 * @file    main.c
 * @brief   the application functions of ADC demo for the
 *          TOSHIBA 'TMPM384' Device Series
 * @version V0.101
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* This demo is to show ADC function with software trigger on Toshiba M384 Ev board   */
/* Change the value of VR2, which is connected to AIN8(pin100), The voltage on it will*/
/*  be measured and used to change the loop blinking speed of the 4 LEDs in board.    */ 
/* The higher the voltage is, the faster the loop blinking speed.                     */

#include "tmpm384_adc.h"
#include "led.h"

/* define the ADC channel used here.the VR2 is connected to AIN8 in this example */
#define ADC_CHANNEL     ADC_AIN8
#define DELAYCONST  20000U

struct mySoftTimer {
    uint32_t flag_TimeUp;
    uint32_t time_Cnt;
}softT;

ADC_Result adResult = { 0U };
uint32_t timeUp = 0U;
uint32_t idx = 0U;
uint32_t cnt = 0U;

WorkState adcState = BUSY;
uint32_t myResult = 0U;

static void softTimer(void); 

int main(void)
{
    /* 1. set ADC clock */
    ADC_SetClk(ADC_HOLD_FIX, ADC_FC_DIVIDE_LEVEL_NONE);

    /* 2. select trigger and AD channel, this time we use sofeware trigger, */
    /*    remember to input with macro TRG_ENABLE() */
    ADC_SetSWTrg(ADC_REG0, TRG_ENABLE(ADC_CHANNEL));

    /* 3. to let ADC module work, we must disable low power mode */
    ADC_SetLowPowerMode(DISABLE);

    /* 4. enable ADC module */
    ADC_Enable();

    /* 5. now start ADC */
    ADC_Start(ADC_TRG_SW);

    /* initialize LEDs on M384 board before display something */
    LED_Init();
    
    /* initialize soft timer */
    softT.time_Cnt = DELAYCONST;
    
    while (1U) {
        /* check ADC module state */
        adcState = ADC_GetConvertState(ADC_TRG_SW);

        if (adcState == DONE) {
            /* read ADC result when it is finished */
            adResult = ADC_GetConvertResult(ADC_REG0);
            
            /* Get the real ADC result without other information */
            /* "/256" is to limit the range of AD value */
            timeUp = 16U - adResult.Bit.ADResult / 256U;
            
            /* software trigger, need to trig it again */
            ADC_Start(ADC_TRG_SW);
        } else {
            /* Do nothing */
        }

        softTimer();  
             
        if(softT.flag_TimeUp) {
            softT.flag_TimeUp = 0U;
            cnt++;
            if( cnt >= timeUp ) {
                cnt = 0U;
                idx++;
                if( idx == 1U ) {
                    LED_On(LED0);
                } else if( idx == 2U ) {
                    LED_On(LED1);
                    LED_Off(LED0);
                } else if( idx == 3U ) {
                    LED_On(LED2);
                    LED_Off(LED1);
                } else if( idx == 4U ) {
                    LED_On(LED3);
                    LED_Off(LED2);
                } else if( idx == 5U ) {
                    idx = 0U;
                    LED_Off(LED3);
                } else {
                    /* Do nothing */
                }
            } else {
                /* Do nothing */
            }  
        } else {
            /* Do nothing */
        } 
    }
}


/**
  * @brief  Simulate a several mS timer. 
  * @param  None
  * @retval None
  */
static void softTimer(void)
{
    if( softT.time_Cnt == 0U ){
        softT.flag_TimeUp = 1U;
        softT.time_Cnt = DELAYCONST;
    } else {
        softT.time_Cnt --;
    }
}

#ifdef DEBUG
void assert_failed(char *file, int32_t line)
{
    while (1U) {
        __NOP();
    }
}
#endif
