/**
 *******************************************************************************
 * @file    main.c
 * @brief   The application functions of DMAC demo for the
 *           TOSHIBA 'TMPM384' Device Series 
 * @version V0.101
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */
/* Include header file */
#include "tmpm384_dmac.h"
#include "tmpm384_dmac_int.h"

#define BUFFER_SIZE  32U
typedef enum { FAILED = 0, PASSED = !FAILED } TestResult;

/* Function declaration */
TestResult Buffercompare(const uint32_t * pBuffer0, const uint32_t * pBuffer1,
                         uint16_t BufferLength);

/* User ram definition */
const uint32_t SRC_Buffer[BUFFER_SIZE] = {
    0x00000000U, 0x11111111U, 0x22222222U, 0x33333333U,
    0x44444444U, 0x55555555U, 0x66666666U, 0x77777777U,
    0x88888888U, 0x99999999U, 0xAAAAAAAAU, 0xBBBBBBBBU,
    0xCCCCCCCCU, 0xDDDDDDDDU, 0xEEEEEEEEU, 0xFFFFFFFFU,
    0xFFFFFFFFU, 0xEEEEEEEEU, 0xDDDDDDDDU, 0xCCCCCCCCU,
    0xBBBBBBBBU, 0xAAAAAAAAU, 0x99999999U, 0x88888888U,
    0x77777777U, 0x66666666U, 0x55555555U, 0x44444444U,
    0x33333333U, 0x22222222U, 0x11111111U, 0x00000000U
};

uint32_t DST_Buffer[BUFFER_SIZE] = { 0U };
volatile TestResult TransferResult = FAILED;

/* main function */
int main(void)
{
    DMAC_InitTypeDef DMAC_InitStruct;
    DMAC_Channel myChannel = DMAC_CHANNEL_1;
    uint32_t LinkAddr;

    SystemInit();

    /* DMA channel1 configuration */
    DMAC_InitStruct.SrcAddr = (uint32_t) SRC_Buffer;
    DMAC_InitStruct.SrcIncrementState = ENABLE;
    DMAC_InitStruct.SrcBitWidth = DMAC_WORD;
    DMAC_InitStruct.SrcBurstSize = DMAC_4_BEATS;
    DMAC_InitStruct.DstAddr = (uint32_t) DST_Buffer;
    DMAC_InitStruct.DstIncrementState = ENABLE;
    DMAC_InitStruct.DstBitWidth = DMAC_WORD;
    DMAC_InitStruct.DstBurstSize = DMAC_4_BEATS;
    DMAC_InitStruct.TxSize = BUFFER_SIZE;
    DMAC_InitStruct.TxDirection = DMAC_MEMORY_TO_MEMORY;

    /* CXCONTROL<I> must be set when use transfer interrupt */
    DMAC_InitStruct.TxINT = ENABLE;

    LinkAddr = 0U;

    __disable_irq();
    /* enable DMA interrupt */
    NVIC_ClearPendingIRQ(INTDMACTC_IRQn);
    NVIC_EnableIRQ(INTDMACTC_IRQn);

    /*Enable DMA circuit */
    DMAC_Enable();
    /* Clear any pending interrupts on the channel */
    DMAC_ClearTxINTReq(myChannel, DMAC_INT_TX_END);
    DMAC_ClearTxINTReq(myChannel, DMAC_INT_TX_ERR);
    /*Initialize DMA channel */
    DMAC_Init(myChannel, &DMAC_InitStruct);
    /* If don't require scatter/gather function, set the DMACCxLLI Register to 0 */
    DMAC_SetLinkedList(myChannel, LinkAddr);
    /*Enable DMA Channel transfer end interrupt */
    DMAC_SetTxINTConfig(myChannel, DMAC_INT_TX_END, ENABLE);
    /*Enable DMA channel */
    DMAC_SetDMAChannel(myChannel, ENABLE);
    __enable_irq();

    /* Wait the end of transmission */
    while (TxEndFlag != DONE) {
        /* Do nothing */
    }

    /* Check whether the transmitted and received data are equal */
    TransferResult = Buffercompare(SRC_Buffer, DST_Buffer, BUFFER_SIZE);

    while (1) {
        /* Do nothing */
    }
}

/* Compare two buffers */
TestResult Buffercompare(const uint32_t * pBuffer0, const uint32_t * pBuffer1,
                         uint16_t BufferLength)
{
    TestResult retval = PASSED;
    while (BufferLength--) {
        if (*pBuffer0 != *pBuffer1) {
            retval = FAILED;
            break;
        } else {
            /* Do nothing */
        }

        pBuffer0++;
        pBuffer1++;
    }

    return retval;
}

#ifdef DEBUG
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif
