/** 
 *******************************************************************************
 * @file    main.c
 * @brief   The application functions of UART0 to UART2 demo for the TOSHIBA
 *          'TMPM384' Device Series 
 * @version V0.101
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

#include "tmpm384_uart.h"
#include <stdio.h>

#define BufSize (sizeof(TxBuffer) / sizeof(*(TxBuffer)))
uint8_t TxBuffer[] = "TMPM384";
uint8_t RxBuffer[BufSize] = { 0U };

const uint8_t NumToBeTx = BufSize - 1U;
uint8_t TxCounter = 0U;
uint8_t RxCounter = 0U;
UART_InitTypeDef myUART;

void SIO_Configuration(TSB_SC_TypeDef * SCx);
void ResetIdx(void);

int main(void)
{
    SIO_Configuration(UART0);
    SIO_Configuration(UART2);

    myUART.BaudRate = (uint32_t) 115200;
    myUART.DataBits = UART_DATA_BITS_8;
    myUART.StopBits = UART_STOP_BITS_1;
    myUART.Parity = UART_NO_PARITY;
    myUART.Mode = UART_ENABLE_RX | UART_ENABLE_TX;
    myUART.FlowCtrl = UART_NONE_FLOW_CTRL;

    UART_Enable(UART0);
    UART_Init(UART0, &myUART);

    UART_Enable(UART2);
    UART_Init(UART2, &myUART);

    NVIC_EnableIRQ(INTTX0_IRQn);
    NVIC_EnableIRQ(INTRX2_IRQn);

    do {
        UART_SetTxData(UART0, (uint32_t) (TxBuffer[TxCounter++]));
        while (RxCounter < NumToBeTx) {
            /* Do nothing */
        }
#ifdef DEBUG
        printf("%s\n", RxBuffer);
#endif
        ResetIdx();
    } while (1);

}

void ResetIdx(void)
{
    uint8_t i = 0U;

    for (i = 0U; i <= RxCounter; i++) {
        RxBuffer[i] = 0U;
    }
    TxCounter = 0U;
    RxCounter = 0U;
}

void SIO_Configuration(TSB_SC_TypeDef * SCx)
{
    if (SCx == TSB_SC0) {
        TSB_PE_CR_PE0C = 1;
        TSB_PE_FR1_PE0F1 = 1;
        TSB_PE_FR1_PE1F1 = 1;
        TSB_PE_IE_PE1IE = 1;
    } else if (SCx == TSB_SC1) {
        TSB_PA_CR_PA5C = 1;
        TSB_PA_FR1_PA5F1 = 1;
        TSB_PA_FR1_PA6F1 = 1;
        TSB_PA_IE_PA6IE = 1;
    } else if (SCx == TSB_SC2) {
        TSB_PD_CR_PD5C = 1;
        TSB_PD_FR1_PD5F1 = 1;
        TSB_PD_FR1_PD6F1 = 1;
        TSB_PD_IE_PD6IE = 1;
    } else if (SCx == TSB_SC3) {
        TSB_PF_CR_PF3C = 1;
        TSB_PF_FR2_PF3F2 = 1;
        TSB_PF_FR2_PF4F2 = 1;
        TSB_PF_IE_PF4IE = 1;
    } else if (SCx == TSB_SC4) {
        TSB_PC_CR_PC6C = 1;
        TSB_PC_FR4_PC5F4 = 1;
        TSB_PC_FR4_PC6F4 = 1;
        TSB_PC_IE_PC7IE = 1;
    }
}

#ifdef DEBUG
void assert_failed(char *file, int32_t line)
{
    printf("Err!");
    while (1) {
        __NOP();
    }
}
#endif
