/**
 *******************************************************************************
 * @file    tmpm384_vltd_int.c
 * @brief   all interrupt request functions definition of VLTD
 *          for the TOSHIBA 'TMPM384' Device Series
 * @version V0.101
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include <stdio.h>
#include "tmpm384_vltd_int.h"
#include "main.h"

/**
  * @brief  The NMI interrupt request function.
  * @param  None
  * @retval None
  */
void NMI_Handler(void)
{
    /* Make sure the NMI is generated by VLTD. */
    if (CG_GetNMIFlag().Bit.VoltageDetection == 1U) {
        LED_On(LED3);
#ifdef DEBUG
        printf("Low voltage!\n");
#endif
        /* Wait for the power supply voltage to be normal. */
        while (VLTD_GetStatus() == 1U) {
            __NOP();
        }
        LED_Off(LED3);
#ifdef DEBUG
        printf("High voltage!\n");
#endif
    } else {
        /* Do nothing */
    }
}
